import os
import harp_agent.settings as settings
import json
import errno
import pwd
import grp
from os.path import exists


def create_config_file():
    if exists(settings.PATH_TO_MS_CONFIG) is False:
        with open(settings.PATH_TO_MS_CONFIG, 'w+') as outfile:
            json.dump({}, outfile)


def create_folders():
    directories = ['/etc/harp-agent']
    uid = pwd.getpwnam("root").pw_uid
    gid = grp.getgrnam("root").gr_gid
    for directory in directories:
        if not os.path.exists(directory):
            os.makedirs(directory)
            os.chown(directory, uid, gid)


def prepare_service_for_first_start():
    create_folders()
    create_config_file()




from microservice_template_core.tools.flask_restplus import api
from flask_restx import Resource
import traceback
from microservice_template_core.tools.logger import get_logger
from flask import request

logger = get_logger()
ns = api.namespace('collector', description='Harp Voice Notification endpoints')


@ns.route('/')
class NotificationStatus(Resource):
    @staticmethod
    @api.response(200, 'Notification has been created')
    @api.response(500, 'Unexpected error on backend side')
    def get():
        """
        Get status of existing Voice Notification in Twilio
        """

        logger.info(
            msg=f"Received request to get status of existing Voice notification via Twilio",
            extra={'tags': {}}
        )
        try:
            status = 'TBD'
            return {'status': status}, 200
        except Exception as exc:
            logger.critical(
                msg=f"Can`t Collect \nException: {str(exc)} \nTraceback: {traceback.format_exc()}",
                extra={'tags': {}})
            return {'msg': 'Exception raised. Check logs for additional info'}, 500




import click
import validators
import harp_agent.settings as settings
import json
import os
import errno


def is_root(ctx, param, value):
    if not value:
        ctx.abort()

    if os.geteuid() == 0:
        return True
    else:
        raise click.BadParameter("""
        ==========================
        You should run command from root user!!!
        ==========================
        """)


def validate_url(ctx, param, value):
    if validators.url(value):
        return value
    else:
        raise click.BadParameter(f'Wrong URL: {value}. Should be in format like - http://system-hostname')


def update_config_file(new_config):
    # Read file
    with open(settings.PATH_TO_MS_CONFIG) as json_file:
        data = json.load(json_file)

    # Prepare config to insert
    unique_integration = f"{new_config['monitoring_system']}_{new_config['integration_name']}"
    if unique_integration in data:
        print(f'Integration - "{unique_integration}" already exist in configuration - {settings.PATH_TO_MS_CONFIG}')
    else:
        data[unique_integration] = new_config

    # Update file
    with open(settings.PATH_TO_MS_CONFIG, 'w') as outfile:
        json.dump(data, outfile)


@click.command()
@click.option('--yes', is_flag=True, callback=is_root, expose_value=False, prompt='Are you running command from root?')
@click.option("--monitoring-system", prompt="Choose monitoring system to configure", help="Monitoring system", type=click.Choice(['zabbix', 'icinga'], case_sensitive=False), required=True)
@click.option("--integration-name", prompt="Choose unique name of your integration", help="Name of your integration", required=True)
@click.option("--url", prompt="URL to your system (http://system-hostname)", help="http://system-hostname", required=True, callback=validate_url)
@click.option("--user", "-u", prompt="API Username", help="Username", required=True)
@click.option("--password", "-p", prompt="API User password", help="Password", required=True, hide_input=True, confirmation_prompt=True)
def agent_add(monitoring_system, integration_name, url, user, password):
    """Simple program that greets NAME for a total of COUNT times."""
    config = {
        'monitoring_system': monitoring_system,
        'integration_name': integration_name,
        'url': url,
        'user': user,
        'password': password
    }
    update_config_file(config)


@click.command()
@click.option("--user", "-u", "user", prompt="API Username", help="Username", required=True)
def agent_update(user):
    """Simple program that greets NAME for a total of COUNT times."""
    print("user: ", user)


def agent_delete(count, name):
    """Simple program that greets NAME for a total of COUNT times."""
    for _ in range(count):
        click.echo(f"Hello, {name}!")



import requests
import json
import traceback
import harp_agent.settings as settings
from microservice_template_core.tools.logger import get_logger
from harp_agent.notifications_collectors.icinga_collector import IcingaCollector
from harp_agent.notifications_collectors.zabbix_collector import ZabbixCollector
import uuid

logger = get_logger()


class PushAlerts(object):
    def __init__(self, monitoring_system, event_id):
        self.monitoring_system = monitoring_system
        self.event_id = event_id

    @staticmethod
    def system_healthy(ms_source):
        try:
            result = requests.get(
                url=ms_source,
                headers={"Content-Type": "application/json"}
            ).status_code
            print(result)
        except Exception as err:
            return False

        if result < 399:
            return True
        else:
            return False

    def push_notifications(self, body):
        url = f"{settings.GATE_HOST}/{self.monitoring_system}"

        logger.info(
            msg=f"Push alerts to Gate Collector for ms={self.monitoring_system} - {url}\nBody: {body}",
            extra={'tags': {'event_id': str(self.event_id)}}
        )

        try:
            result = requests.post(
                url=url,
                data=json.dumps(body),
                headers={"Content-Type": "application/json", "EVENT-ID": self.event_id}
            ).json()

            logger.info(
                msg=f"Receive response from Gate Collector - {result}",
                extra={'tags': {'event_id': str(self.event_id)}}
            )
        except Exception as err:
            result = {}
            logger.error(
                msg=f"Gate Collector is unreachable: {err}",
                extra={'tags': {'event_id': str(self.event_id)}}
            )

        return result

    def main(self):
        result = None
        ms_alerts = {}

        if self.monitoring_system == 'zabbix':
            for ms_source in settings.ZABBIX_SYSTEMS:
                if self.system_healthy(ms_source['url']):
                    logger.info(
                        msg=f"Start scraping triggers from ms_source={ms_source['url']}",
                        extra={'tags': {'event_id': str(self.event_id)}}
                    )
                    collector = ZabbixCollector(ms_source=ms_source, event_id=self.event_id)
                    ms_alerts_raw, alerts_count = collector.main()
                    logger.info(
                        msg=f"Finish scraping triggers. Result - ms_alerts_raw: {ms_alerts_raw}, alerts_count: {alerts_count}",
                        extra={'tags': {'event_id': str(self.event_id)}}
                    )
                    if ms_alerts_raw or alerts_count == 0:
                        ms_alerts[ms_source] = ms_alerts_raw
                    result = self.push_notifications(ms_alerts)
                else:
                    logger.error(
                        msg=f"Monitoring system is unreachable: ms_source: {ms_source['url']}",
                        extra={'tags': {'event_id': str(self.event_id)}}
                    )

        if self.monitoring_system == 'icinga':
            for ms_source in settings.ICINGA_SYSTEMS:
                logger.info(
                    msg=f"Start scraping alerts from ms_source={ms_source['url']}",
                    extra={'tags': {'event_id': str(self.event_id)}}
                )
                collector = IcingaCollector(ms_source=ms_source, event_id=self.event_id)
                ms_alerts_raw, alerts_count = collector.main()
                logger.info(
                    msg=f"Finish scraping alerts. Result - ms_alerts_raw: {ms_alerts_raw}, alerts_count: {alerts_count}",
                    extra={'tags': {'event_id': str(self.event_id)}}
                )
                if ms_alerts_raw or alerts_count == 0:
                    ms_alerts[ms_source['url']] = ms_alerts_raw
                result = self.push_notifications(ms_alerts)

        return result


def notification_scraper(monitoring_system):
    event_id = str(uuid.uuid4())
    try:
        logger.info(
            msg=f"Scheduled scraping triggers from {monitoring_system}",
            extra={'tags': {'event_id': str(event_id)}}
        )
        notifications = PushAlerts(monitoring_system, event_id)
        print(json.dumps(notifications.main()))
    except Exception as err:
        logger.exception(msg=err)


import os

# General settings
NOTIFICATIONS_SCRAPE_INTERVAL_SECONDS = os.getenv('NOTIFICATIONS_SCRAPE_INTERVAL', 5)
PATH_TO_MS_CONFIG = os.getenv('PATH_TO_MS_CONFIG', '/etc/harp-agent/config.json')

# Collector settings
GATE_HOST = os.getenv('GATE_HOST', 'http://127.0.0.1:8081/api/v1/gate-collector')

# Example: {'url': 'http://127.0.0.1:8084', 'user': 'Admin', 'password': 'zabbix'}
ZABBIX_SYSTEMS = []

# Example: {'url': 'http://127.0.0.1', 'user': 'root', 'password': 'd0c87be02fdb79f0'}
ICINGA_SYSTEMS = []





import requests
import json
import traceback
from microservice_template_core.tools.logger import get_logger
import urllib3

logger = get_logger()
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class IcingaCollector(object):
    def __init__(self, ms_source, event_id):
        self.ms_source = ms_source
        self.alerts_count = 0
        self.event_id = event_id
        self.headers = {
            'Accept': 'application/json',
            'X-HTTP-Method-Override': 'GET'
        }

    @staticmethod
    def ms_notification_output(result_body):
        output = {
            "NOTIFICATIONAUTHORNAME": "",
            "SERVICENOTES": "",
            "SERVICENAME": result_body['attrs']['name'],
            "SERVICEDOWNTIME": "0",
            "SERVICEACTIONURL": "",
            "HOSTNAME": result_body['joins']['host']['name'],
            "HOSTADDRESS": result_body['joins']['host']['address'],
            "SEVERITY": result_body['attrs']['severity'],
            "SERVICENOTESURL": "",
            "LASTSTATETYPE": result_body['attrs']['state_type'],
            "NOTIFICATIONTYPE": "PROBLEM",
            "HOSTSERVICE": "",
            "HOSTROLE": "",
            "ICINGASERVER": "",
            "SERVICESTATE": result_body['attrs']['state'],
            "SERVICEOUTPUT": result_body['attrs']['last_check_result']['output'],
            "SERVICESTATETYPE": "",
            "NOTIFICATIONCOMMENT": "",
            # "PROCEDURE_ID": "2387",
            "PROBLEMTYPE": result_body['type'],
            "SERVICEGRAPH": ""
        }

        return output

    @staticmethod
    def ms_unique_name(result_body):
        unique_name = f"other:::{result_body['joins']['host']['name']}:::{result_body['attrs']['name']}"

        return unique_name

    def collect_services(self):
        alerts_per_host = {}

        request_url = "{0}:5665/v1/objects/services".format(self.ms_source['url'])
        data = {
            "joins": ["host.name", "host.address"],
            "filter": "service.state != 0"
        }

        response = requests.post(
            request_url,
            headers=self.headers,
            auth=(self.ms_source['user'], self.ms_source['password']),
            verify=False,
            data=json.dumps(data),
            timeout=5
        ).json()

        for result in response['results']:
            notification_body = self.ms_notification_output(result_body=result)
            ms_unique_name = self.ms_unique_name(result_body=result)
            alerts_per_host[ms_unique_name] = notification_body

            self.alerts_count += 1

        logger.info(
            msg=f"Collected list of alerts from Icinga:\n{alerts_per_host}",
            extra={'tags': {'event_id': str(self.event_id)}}
        )

        return alerts_per_host

    def collect_alerts(self):
        alerts_per_host = self.collect_services()

        request_url = "{0}:5665/v1/objects/hosts".format(self.ms_source['url'])
        data = {
            "attrs": ["last_state"],
            "filter": "host.last_state != 0"
        }

        response = requests.post(
            request_url,
            headers=self.headers,
            auth=(self.ms_source['user'], self.ms_source['password']),
            verify=False,
            data=json.dumps(data),
            timeout=5
        ).json()

        if len(response['results']) == 0:
            return alerts_per_host, self.alerts_count

        for result in response['results']:
            notification_body = self.ms_notification_output(result_body=result)
            ms_unique_name = self.ms_unique_name(result_body=result)
            alerts_per_host[ms_unique_name] = notification_body

            self.alerts_count += 1

        return alerts_per_host, self.alerts_count

    def main(self):
        return self.collect_alerts()




from setuptools import setup, find_packages
import sys

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 7)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version of Harp Agent requires Python {}.{}, but you're trying to
install it on Python {}.{}.
Please install Python version >=3.7
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)


with open('requirements.txt') as f:
    requirements = f.read().splitlines()


tests_require = ['test'],


setup(
    name='harp-agent',
    python_requires='>3.7.0',
    version='1.0.2',
    description="Harp Agent",
    url='',
    include_package_data=True,
    author='',
    author_email='',
    classifiers=[
    ],
    keywords=[],
    packages=find_packages(),
    install_requires=requirements,
    tests_require=tests_require,
    entry_points={
        'console_scripts': [
            'harp-agent = harp_agent.app:main',
            'harp-agent-add = cmd_command.agent_configuration:agent_add',
            'harp-agent-update = cmd_command.agent_configuration:agent_update',
            'harp-agent-delete = cmd_command.agent_configuration:agent_delete',
        ]
    },
    zip_safe=False,
    cmdclass={},
    package_files=[
        ('/etc/init.d', [
            'data/init-script/harp-agent'
        ])
    ]
)



import time
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import harp_agent.settings as settings
import json


class MyHandler(FileSystemEventHandler):
    def on_modified(self, event):
        print(f'event type: {event.event_type}  path : {event.src_path}')

        with open(settings.PATH_TO_MS_CONFIG) as json_file:
            data = json.load(json_file)

        for system_name, system_body in data.items():
            system_payload = {
                'url': system_body['url'],
                'user': system_body['user'],
                'password': system_body['password']
            }
            if system_body['monitoring_system'] == 'zabbix':
                settings.ZABBIX_SYSTEMS.clear()
                settings.ZABBIX_SYSTEMS.append(system_payload)

            if system_body['monitoring_system'] == 'icinga':
                settings.ICINGA_SYSTEMS.clear()
                settings.ICINGA_SYSTEMS.append(system_payload)

        print(f"ZABBIX_SYSTEMS: {settings.ZABBIX_SYSTEMS}")
        print(f"ICINGA_SYSTEMS: {settings.ICINGA_SYSTEMS}")


def update_configuration():
    event_handler = MyHandler()
    observer = Observer()
    observer.schedule(event_handler, path=settings.PATH_TO_MS_CONFIG, recursive=False)
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()
